.ifdef HIGHVER
.export __LOADADDR__ = $0801
.endif

.ifdef LOWVER
.export __LOADADDR__ = $0801
.endif

.ifdef CODEVER
.export __LOADADDR__ = $C000
.endif

; Here we define the address bytes needed by C64 so it knows where it should load it
.segment "LOADADDR"
.byte .LOBYTE( __LOADADDR__ ), .HIBYTE( __LOADADDR__ )
.segment "CODE"

.ifndef PARTIAL
CMDCLRS:
CMDAT:
CMDTCLR:
CMDBCLR:
CMDNOTON: 
CMDVVAL:
CMDNOTE: 
CMDLENV:
CMDFORM: 
CMDUSRT:
CMDCDEF:
CMDGR: 
CMDCLRG:
CMDORD:
CMDPPOS:
CMDLI:
CMDGETPRINT:
CMDSQRCLR:
CMDSPOS:
CMDSDEF:
CMDIFCLOSE:
CMDEXPAND:
CMDWAITFOR:
CMDDPOKE:
CMDMPOKE:
CMDSTOPON:
CMDRESTORETO:

;*** REGISTER HERE 
.endif


.ifdef HIGHVER
basic:
     .word $80A, 10  ;pointer, line number
     .byte $9e
     .byte "2061"
     .word 0          ;basic line end
     .byte 0
startprog:
dest=$C000
hiinitstart:
    lda #.LOBYTE(main)  
    sta $FB
    lda #.HIBYTE(main)  
    sta $FC
    lda #.LOBYTE(dest)  
    sta $FD
    sta $14
    lda #.HIBYTE(dest)  
    sta $FE
    sta $15
    ldx #.HIBYTE(endcode-main)  
    
    beq rlclastpg      ; But if it is already zero, skip first part
    ldy #0             ; Set Y counter to zero, copy in 256 bytes blocks
rlclp:                 ; Loop that copyes 256 bytes blocks
    lda ($FB),y        ; Take source byte
    sta ($FD),y        ; Store to destination          
    iny                ; Increment small pointer
    bne rlclp          ; Wrapped to zero?
    inc $FC            ; Increment higher bytes of source pointer
    inc $FE            ; And destination pointer
    dex                ; But decrement block counter
    bne rlclp          ; And repeat until zero
rlclastpg:             ; Remaining bytes
    ldx #.LOBYTE(endcode-main)    ; X is now byte counter 
    beq rlcend         ; If it is yero we have finished
lastpgrlclp:           ; Last block loop
    lda ($FB),y        ; Take source byte
    sta ($FD),y        ; Store to destination byte
    iny                ; increment small pointer
    dex                ; Decrement byte counter
    bne lastpgrlclp    ; Until zero
rlcend:
    jsr init           ; initialize
    jmp $A644           ; NEW
init:
    jmp ($0014) 
main:
   lda $14            ; SYS command stores address here
   clc                
   adc #(testex-main) ; Make it relocatable  
   sta $308           ; Lower byte of IGONE vector
   lda $15
   adc #0             ; 16 bit addition
   sta $309           ; Higher byte of IGONE vector
   rts
hiinitend:
.endif

.ifdef LOWVER
basic:
     .word $80A, 10  ;pointer, line number
     .byte $9e
     .byte "2061"
     .word 0          ;basic line end
     .byte 0
loinitstart:
lomain:
   lda $14            ; SYS command stores address here
   clc                
   adc #(loinitend-lomain) ; Make it relocatable  
   sta $308           ; Lower byte of IGONE vector
   lda $15
   adc #0             ; 16 bit addition
   sta $309           ; Higher byte of IGONE vector
   lda #1             ; Change beginning of BASIC program low byte
   sta 43              
   lda #(((filler-basic)/$100+1)+8)  
   sta 44             ; Change beginning of BASIC program high byte
   jsr $A659          ; reset execute pointer and do CLR
   jsr $A533          ; rebuild BASIC line chaining
   jmp $a7b1          ; Interpreter loop
loinitend:
.endif

.ifdef CODEVER
main:
   lda $14            ; SYS command stores address here
   clc                
   adc #(testex-main) ; Make it relocatable  
   sta $308           ; Lower byte of IGONE vector
   lda $15
   adc #0             ; 16 bit addition
   sta $309           ; Higher byte of IGONE vector
   rts
.endif

testex:                ; Entry point before every new instruction
    jsr $0073          ; call CHRGET. Takes new character from input
    cmp #$21           ; Is it "!"?
    beq getletter      ; If yes, it might be a new command
    jmp $A7E7          ; Continue interpreting BASIC if not, without getting new char
getletter:
    ldy #0             ; Required due to addressing (here we can not use self modification trick)
    inc $7A            ; Increment pointer to the next char
    bne makehash1      ; If zero
    inc $7B            ; update higher byte
makehash1:
    lda ($7A),y        ; First byte (ASCII or token)
    asl                ; Shifted for diversification
    inc $7A            ; Again increment pointer to the next char
    bne makehash2      ; If zero
    inc $7B            ; Update higher byte
makehash2:
    adc ($7A),y        ; And add the second byte


commands:

.ifdef CMDCLRS
home:                  ; !CLRS  clear screen and home cursor
    cmp #$8C           ; hash of !CLRS
    bne endhome        ; if not try other letter
    lda #$93           ; Clear screen ASCII code
    jsr $FFD2          ; Kernal output character routine
    jmp $A7E4          ; Continue interpreting BASIC with getting new character
endhome:
.endif

.ifdef CMDAT
at:                    ; !AT row,col set textual cursor position
    cmp #$D6           ; hash of AT
    bne endat          ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    stx $02            ; Store it to unused zero page location 2
    jsr $B7F1          ; Check for comma then get byte parameter into X
    ldy $02            ; get first parameter in y
    cpy #40            ; check x coordinate 0-39
    bcs atrangeerr     ; if greater or equal carry=1
    cpx #25            ; check y coordinate 0-24
    bcs atrangeerr     ; if greater or equal carry=1
    clc                ; clear carry if setting cursor. well, it is already, but to be safe
    jsr $FFF0          ; read/set X,Y cursor position
    jmp $A7AE          ; jump to main interpreter loop to check new command
atrangeerr:
    jmp $B248          ; Illegal quantity error
endat:
.endif

.ifdef CMDTCLR

color:                 ; !TCLR textcolor   set textual color
    cmp #$44           ; hash of TCLR
    bne endcolor       ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    stx $0286          ; Store text color
    jmp $A7AE          ; jump to main interpreter loop to check new command
endcolor:
.endif

.ifdef CMDBCLR

backgnd:               ; !BCLR papercolor,bordercolor   set colors in text mode
    cmp #$20           ; hash of BCLR
    bne endbackgnd     ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    stx $02            ; Store it to unused zero page location 2
    jsr $B7F1          ; Check for comma then get byte parameter into X
    ldy $02            ; get first parameter in y
    stx $D020          ; Border color
    sty $D021          ; Background color
    jmp $A7AE          ; jump to main interpreter loop to check new command      
endbackgnd:
.endif

.ifdef CMDNOTON

quieten:               ; !NOTON  reset music chip
    cmp #$E2           ; hash of NOTON
    bne endquieten     ; if not try other letter
    lda #$0            ; Clear all SID chip registers
    ldx #24            ; There are 24 registers
mclr:
    sta $D400,x        ; put zero to each
    dex                ; Decre ment counter
    bne mclr           ; Return to loop
    sta $d400          ; Last register
    jmp $A7E4          ; Continue interpreting BASIC with getting new character
endquieten:
.endif

.ifdef CMDVVAL

volume:                ; !VVAL vol 0-15  set master volume
    cmp #$71           ; hash of VVAL
    bne endvolume      ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    txa
    and #$0F
    sta $D418          ; Put to volume register
    asl a              ; Also adjust sustain at all three canals (in upper nibble)
    asl a
    asl a
    asl a
    sta $D406          ; Sustain/release all three canals
    sta $D40D
    sta $D414
    jmp $A7AE          ; jump to main interpreter loop to check new command
endvolume:

.endif

.ifdef CMDNOTE

note:                  ; !NOTE canal,freq
    cmp #$96           ; Hash of NOTE
    bne endnote        ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    lda #$F9           ; -7
nmul7:
    clc                ;
    adc #$7            ; +7
    dex                ; decrement
    bne nmul7
    cmp #15            ; if canal>3 error
    bcs noteerr
    sta $2             ; Now address 2 keeps parameter-1 multiplied by 7, relative register of chanel
    jsr $AEFD          ; check for comma
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI ; maybe * 17 to use Hertz?
    ldx $2             ; relative positon of canal registers 
    pha                ; save A before checking for zero
    sty $2             ; test for zero in a and y
    ora $2             ; both register if 0, do not enable out
    beq silence        ; then jump to silence
    pla                ; restore A with low frequence
    sta $D401,x        ; hifreq
    tya
    sta $D400,x        ; lofreq 
    lda #$1            ; enable out
writegate:
    ora #$10           ; triangle wave
    sta $D404,x        ; gate
    jmp $A7AE          ; jump to main interpreter loop to check new command      
silence:
    pla                ; A is still zero on stack so
    beq writegate      ; we can use relative jump here
noteerr:
    jmp $B248          ; Illegal quantity error
endnote:

.endif

.ifdef CMDLENV

envelope:              ; !LENV canal,attack,decay,sustain,release
    cmp #$DD           ; Hash of LENV
    bne endenvelope    ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    lda #$F9           ; -7
emul7:
    clc                ;
    adc #$7            ; +7
    dex                ; decrement
    bne emul7
    cmp #15            ; if canal>3 error
    bcs enverr
    sta $2             ; Now address 2 keeps parameter-1 multiplied by 7, relative register of chanel
    lda #1             ; Make loop twice
    sta $F9
twonibpars:
    jsr $B7F1          ; Check for comma then get byte parameter into X
    txa                ; To accumulator
    asl a              ; just higher nibble (attack or sustain)
    asl a
    asl a
    asl a
    sta $FB            ; Put to temp location
    jsr $B7F1          ; Check for comma then get byte parameter into X
    txa                ; To accumulator
    and #$0F           ; Decay is in lower nibble
    ora $FB            ; Combine nibbles
    pha                ; store attack/decay or sustain/release on stack
    lsr $F9            ; Carry is 1 after first pass, 0 after second pass
    bcs twonibpars     ; if required get remaining two parameters
    ldx $2             ; restore canal delta
    pla
    sta $D406,x        ; sustain/release
    pla
    sta $D405,x        ; attack/deecay
    jmp $A7AE          ; jump to main interpreter loop to check new command      
enverr:
    jmp $B248          ; Illegal quantity error
endenvelope:

.endif

.ifdef CMDFORM

wave:                  ; !FORM canal,freq,type,pulse
    cmp #$50           ; hash of FORM
    bne endwave        ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    lda #$F9           ; -7
wmul7:
    clc                ;
    adc #$7            ; +7
    dex                ; decrement
    bne wmul7
    cmp #15            ; if canal>3 error
    bcs waveerr
    sta $2             ; Now address 2 keeps parameter-1 multiplied by 7, relative register of chanel
    jsr $AEFD          ; check for comma
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI ; maybe * 17 to use Hertz?
    ldx $2
    sta $D401,x        ; hifreq
    tya
    sta $D400,x        ; lofreq 

    jsr $B7F1          ; Check for comma then get byte parameter into X
    txa
    ldx $2
    sta $D404,x        ; waveform code

    jsr $AEFD          ; check for comma
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI 
    ldx $2
    sta $D403,x        ; hipulse
    tya
    sta $D402,x        ; lopulse

    jmp $A7AE          ; jump to main interpreter loop to check new command      

waveerr:
    jmp $B248          ; Illegal quantity error
endwave:

.endif

.ifdef CMDUSRT

usertext:              ; !USRT
    cmp #$C3           ; Hash of USRT
    bne endusertext    ; if not try other letter
    lda $DD02          ; Read the data direction register
    ora #3             ; Bits 0 and 1 of DD00 are for output
    sta $DD02          ; Change the data direction register
    lda $DD00          ; Bit 0..1 of CIA1 Select the position of the VIC-memory
    and #$FC           ; Switch to bank 3 - 00=$C000 
    sta $DD00          ; Write CIA
    lda #$38           ; Set text video memory to $CC00, font to $E000
    sta $D018          ; Memory pointer register
    sei                ; Disable interrupt. We need to switch off u/i MAP
    lda $01            ; 6510 register 1 defines selected memory.
    pha                ; Remember previous memory configuration
    and #$FB           ; Put character ROM in address space
    sta $01            ; Write to 6510 register 1 to redefine memory
    ldy #$00           ; reset x and y for our loop
    ldx #16            ; 16*256=4096    
    sty $FB            ;set our source memory address to copy from, $D000
    lda #$D0
    sta $FC    
    sty $FD            ; set our destination memory to copy to, $E000, WRAM
    lda #$E0
    sta $FE
copyfont:
    lda ($FB),Y        ; indirect index source memory address, starting at $00
    sta ($FD),Y        ; indirect index dest memory address, starting at $00
    iny                ; increment index in page
    bne copyfont       ; loop until our dest goes over 255
    inc $FC            ; increment high order source memory address, starting at $D0
    inc $FE            ; increment high order dest memory address, starting at $E0
    dex
    bne copyfont       ; if we're not there yet, loop
    pla                ; restore memory configuration
    sta $01            ; Write to 6510 register 1 to redefine memory
    cli                ; Enable interrupt
    lda #$CC           ; Inform Kernal about text memory for regular mode
    sta $288           ; Location for this text screen is at CC00
    jmp $A7E4          ; Continue interpreting BASIC with getting new character

endusertext:
.endif

.ifdef CMDCDEF

font:                  ; !CDEF addr  or !CDEF..##....
    cmp #$1C           ; hash of CDEF
    bne endfont        ; if not try other letter
    jsr $0073          ; Scan for next symbol
    cmp #$23           ; ASCII of "#"
    beq defchar
    cmp #$2E           ; ASCII of "."
    beq defchar
startfont:
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI
    sty $FB            ; Zero page location ($FB-$FC) keeps font address
    sta $FC
fontfin:
    jmp $A7AE          ; jump to main interpreter loop to check new command       
defchar:
    tax                ; Remember first symbol
    lda #8             ; Byte counter
    sta $2             ; At location 2
    lda #0             ; Shifted byte
    sta $FD            ; At location FD
    txa                ; Get first symbol (# or . back)
charlp:
    cmp #$2E           ; ASCII of "."
    beq chzero         ; . encodes binary 0
    cmp #$23           ; ASCII of "#"
    bne fontfin        ; # encodes binary 1
chone:
    sec                ; Stuff one from the right side
    rol $FD            ; of the shifted byte
    bcc nextbit        ; Last one is zero as maximal cycle is 8
chzero:
    clc                ; Stuff zero from the right side
    rol $FD
nextbit:
    jsr $0073          ; Scan for next symbol
    dec $2             ; Decrement loop and
    bne charlp         ; Repeat for 8 symbols
    pha                ; Sprites need 24 symbols, characters just 8, so keep the symbol
    ldy #0             ; y=0
    lda $FD            ; Get into A complete byte
    sta ($FB),y        ; Store it at current location
    inc $FB            ; Increment the location
    bne coll8          ; If zero
    inc $FC            ; increment higher byte
coll8:
    pla                ; Get '.' or '#' symbol back
    clc
    bcc defchar        ; jump to loop
endfont:
.endif

.ifdef CMDGR

graphical:             ; !GR n  turn on 320x200 (n<>1) or 160x200 (n=1) graphical mode
    cmp #$E0           ; Hash of GR
    bne endgraphical   ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    lda #$C8           ; Disable multicolor, 40 characters, normal horizontal        
    dex                ; Test if command parameter is 1
    bne setmcolor      ; If not, switch off multicolor
    ora #$10           ; Set multicolor bit
setmcolor:
    sta $D016          ; Enaable/disable multicolor and scroll
    lda $DD02          ; Read the data direction register
    ora #3             ; Bits 0 and 1 of DD00 are for output
    sta $DD02          ; Change the data direction register
    lda $DD00          ; Bit 0..1 of CIA1 Select the position of the VIC-memory
    and #$FC           ; Switch to bank 3 - 00=$C000 
    sta $DD00          ; Write CIA
    lda #$38           ; Set color map to $CC00, screen to $E000 
    sta $D018          ; Memory pointer register
    lda $D011          ; And turn on bitmap
    ora #$20           ; by setting bit 5, BMM
    sta $D011          ; of the bitmap mode
    jmp $A7AE          ; jump to main interpreter loop to check new command      
endgraphical:
.endif

.ifdef CMDCLRG

zerograph:             ; !CLRG paper,ink   Zero graphical video with initial colors
    cmp #$80           ; Hash of CLRG
    bne endzerograph   ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    stx $02            ; Store it to unused zero page location 2
    jsr $B7F1          ; Check for comma then get byte parameter into X
    txa                ; put to acc
    asl a              ; shift foreground color to the higher nibble
    asl a
    asl a
    asl a
    ora $02            ; stuff background color in lower nibble. A now has attribute
    ldy #0             ; Y is enough to fill 256 attributes
    sty $14            ; Locations $14-$15 will point to color attributes at CC00
    ldx #$CC
colorlp:
    stx $15            ; Change higher byte of attribute pointer
draw_attrb:
    sta ($14),Y        ; set attribute at attribute pointer+Y
    iny                ; Next attribute        
    bne draw_attrb     ; until Y wraps to 0
    inx                ; Increment higher attribute pointer byte
    cpx #$D0           ; Did we reach I/O area
    bne colorlp        ; If not repeat loop
    lda #$0            ; Now similar loop to fill bitmap with zeros
    sta $14            ; Locations $14-$15 will point to bitmap at E000           
    ldx #$E0
clrbitmap:             ; Clear bitmap
    stx $15            ; Change higher byte of bitmap pointer
clrgrp:                ; Clearing group of 256 bytes
    sta ($14),Y        ; zero byte at bitmap pointer+Y        
    iny                ; next bitmap byte
    bne clrgrp         ; until Y wraps to 0
    inx                ; After we reach end of bitmap, it is also end of memory
    bne clrbitmap      ; If not, repeat with next 256 bytes
    jmp $A7AE          ; jump to main interpreter loop to check new command      
endzerograph:
.endif

.ifdef CMDORD

ordinary:              ; !ORD  return to the ordinary text mode
    cmp #$A5           ; Hash of ORD
    bne endordinary    ; if not try other letter
    lda $DD02          ; Read the data direction register
    ora #3             ; Bits 0 and 1 of DD00 are for output
    sta $DD02          ; Change the data direction register
    lda $DD00          ; Bit 0..1 of CIA1 Select the position of the VIC-memory
    ora #$03           ; Switch to bank 0 - 11=$0000 
    sta $DD00          ; Update CIA1
    lda #$15           ; Screen mem --> $0400
    sta $D018
    lda #$C8           ; Disable multicolor, 40 characters, normal horizontal 
    sta $D016          ; Screen control register #2
    lda $D011          ; Bit 5 of mode register turns bitmap on/off
    and #$DF           ; Reset it
    sta $D011          ; Deactivate bitmap
    lda #4             ; Inform Kernal about text memory for regular mode
    sta $288           ; Location text screen is at $0400, store high byte
    jmp $A7E4          ; Continue interpreting BASIC with getting new character
endordinary:
.endif

.ifdef CMDPPOS
.ifdef CMDLI

line:                  ; !LI x1,y1,x2,y2,mode
    cmp #$E1           ; Hash of LI
    bne endline2       ; if not try other letter
    jsr $0073          ; Scan for next symbol
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI
    sty $FA            ; Store it to zero page location ($FA-$FB) initially keep X coordinate
    sta $FB
    cmp #$01           ; Compare it with 1*256+64=320 , first high bytes
    bne ltestx1        ; if they are not equal, go to check carry                     
    cpy #64            ; now test lower byte with 64
ltestx1:
    bcs lineparerr     ; if x coordinate is between 0 and 319 continue
getliney1:
    jsr $B7F1          ; Check for comma then get byte parameter into X
    stx $FC            ; Store it to unused zero page location ($FC) initially keep Y coordinate
    txa
    cmp #200           ; It must be below 200
    bcs lineparerr     ; Go to general quantity error
    jsr $AEFD          ; check for comma
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI
    sty $5A            ; Store it to unused zero page location ($FA-$FB) initially keep X coordinate
    sta $5B
    cmp #$01           ; Compare it with 1*256+64=320 , first high bytes
    bne ltestx2        ; if they are not equal, go to check carry
    cpy #64            ; now test lower byte with 64
ltestx2:
    bcs lineparerr     ; if x coordinate is between 0 and 319 continue

getliney2:
    jsr $B7F1          ; Check for comma then get byte parameter into X
    stx $5C            ; Store it to unused zero page location ($FC) initially keep Y coordinate
    cpx #200           ; It must be below 200
    bcs lineparerr     ; Go to general quantity error

    jsr $B7F1          ; Check for comma then get byte parameter into X
    stx $5D
    clc
    bcc setdx
lineparerr:
    jmp $B248          ; Illegal quantity error
endline2:	  
    bne endline1       ; Ugly bridge

; 1510 REM Inputs are X1(FA,FB), Y1 (FC), X2 (5A,5B), Y2 (5C): Destroys value of X1, Y1
; DX (5E,5F), SX (57,58) , DY (55,56), SY (6A,6B), ER (6c,6d), E2 (6E,6F)
setdx:                 ; DX = X2 - X1 : SX = 1 : IF DX<0 THEN SX = -1 : DX = -DX
    lda $5a            ; X2 low
    sec
    sbc $FA            ; X2-X1 low
    sta $5E            ; DX low 
    lda $5b            ; X2 low
    sbc $FB            ; X2-X1 low
    sta $5F            ; DX high
    ldx #$01           ; SX=1             
    stx $57            ; SX low
    dex             
    stx $58            ; SX high
    and #$80           ; Check DX sign
    beq setdy          ; If Dx is not negative skip the following part
    dex                ; SX=-1            
    stx $57            ; SX low
    stx $58            ; SX high

    inx                ; X=0
    txa                ; Now 0
    sec                ; Find 2KK of 5E
    sbc $5E            ; DX
    sta $5E            ; 0-dx low
    txa                ; 0-
    sbc $5F            ; DX
    sta $5F            ; 0-dX hi

setdy:                 ; DY = Y2 - Y1 : SY = 1 : IF DY<0 THEN SY = -1 : DY = -DY
    lda $5c            ; Y2
    sec
    sbc $FC            ; Y2-Y1 lo
    sta $55            ; DY low
    lda #0             ; Prepare for positive number
    sbc #0             ; After carry subtracted it will be 0 of FF
    sta $56            ; DY hi
    ldx #$01           ; SY=1             
    stx $6a            ; SY low
    dex                ; X=0       
    lda $56            ; Check DY sign
    and #$80
    beq seter          ; Skip if positive
    txa                ; X=0 here so put 0 to A
    sec                ; Find 2KK of 55
    sbc $55            ; DY
    sta $55            ; 0-dy low
    txa                ; X=0 here so put 0 to A
    sbc $56            ; DY
    sta $56            ; 0-dy hi

    dex                ; Now SY=-1            
    stx $6a            ; SY low
    inx                ; X=0 a

seter:                 ; ER = -DY : IF DY-DX <0 THEN ER = DX
    sec                ; Find 2KK of 62
    txa                ; X=0 here so put 0 to A
    sbc $55            ; DY
    sta $6c            ; 0-dy low
    txa                ; X=0 here so put 0 to A
    sbc $56            ; DY
    sta $6d            ; 0-dy hi

    sec
    lda $55            ; DY low byte
    sbc $5e            ; DX low byte
    lda $56            ; DY high byte
    sbc $5F            ; DX high byte
    and #$80           ; Check DY-DX sign
    beq halfer

    lda $5e            ; low dx
    sta $6c            ; to low er           
    lda $5f            ; high dx
    sta $6d            ; to high er
halfer:                ; ER = INT(ER / 2)
    lda $6d            ; copy high byte of ER into A
    asl                ; copy sign bit of A into carry (shorter than CMP)
    ror $6d
    ror $6c

    clc
    bcc plotinline
endline1:
    bne endline        ; Another ugly bridge

bresenloop:                       ; 1570 IF X1 = X2 AND Y1 = Y2 THEN RETURN
    lda $FA
    cmp $5A
    bne sete2
    lda $FB
    cmp $5B
    bne sete2
    lda $FC
    cmp $5C
    bne sete2
    jsr $B391          ; clear FP registers that we used for temporary
    jmp $A7AE          ; jump to main interpreter loop to check new command      
sete2:                 ; 1580 E2 = ER
    lda $6c            ; low er
    sta $6e            ; to low e2           
    lda $6d            ; high er
    sta $6F            ; to high e2

                       ; 1590 IF -(E2+DX) < 0 THEN ER = ER - DY : X1 = X1 + SX
    clc
    lda $6e            ; E2 low byte
    adc $5e            ; DX low byte
    lda $6F            ; E2 high byte
    adc $5F            ; DX high byte
    and #$80           ; Check DY-DX sign
    bne isdy
    
    lda $6c            ; ER low
    sec
    sbc $55            ; ER-DY low
    sta $6c            ; ER low 
    lda $6d            ; ER high
    sbc $56            ; ER-DY high
    sta $6d            ; ER high

    lda $FA            ; X1 low
    clc
    adc $57            ; X1+SX low
    sta $FA            ; X1 low 
    lda $FB            ; X1 high
    adc $58            ; X1+SX high
    sta $FB            ; X1 high


isdy:                  ; 1600 IF E2-DY < 0 THEN ER = ER + DX : Y1 = Y1 + SY
    sec
    lda $6e            ; E2 low byte
    sbc $55            ; DY low byte
    lda $6F            ; E2 high byte
    sbc $56            ; DY high byte
    and #$80           ; Check E2-DY sign
    beq plotinline
    lda $6c            ; ER low
    clc
    adc $5e            ; ER+Dx low
    sta $6c            ; ER low 
    lda $6d            ; ER high
    adc $5f            ; ER+Dx high
    sta $6d            ; ER high
    lda $FC            ; Y1 
    clc
    adc $6a            ; Y1+SY low
    sta $FC            ; Y1 w 


                       ; 1610 PLOT X0,Y0: GOTO 1560
plotinline:            ; Get plot color
    ldx $5D
    lda #0             ; Store 0 on stack before dopixel from !P so that it returns to retline
    pha
    beq dopixel        ; It is zero, so jump to pixel plotting
retline:
    clc
    bcc bresenloop    
endline:
.endif


plot:                  ; !PPOS x,y,mode
    cmp #$59           ; Hash of PPOS  ****
    bne endplot1       ; if not try other letter
    jsr $0073          ; Scan for next symbol
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI
    sty $FA            ; Store it to unused zero page location ($FA-$FB) initially keep X coordinate
    sta $FB
    cmp #$01           ; Compare it with 1*256+64=320 , first high bytes
    bne testx1         ; if they are not equal, go to check carry
    cpy #64            ; now test lower byte with 64
testx1:
    bcs plotparerr       ; if x coordinate is between 0 and 319 continue
getploty:
    jsr $B7F1          ; Check for comma then get byte parameter into X
    stx $FC            ; Store it to unused zero page location ($FC) initially keep Y coordinate
    cpx #200           ; It must be below 200
    bcc getcolor       ; If yes, take next parameter
plotparerr:
    jmp $B248          ; Illegal quantity error
retline1:
    beq retline        ; Backward bridge to LINE routine
endplot1:
    bne endplot        ; Plot routine is longer than 128 bytes, we need this bridge to keep relocatable
getcolor:
    jsr $B7F1          ; Check for comma then get byte parameter into X

    lda #$1            ; 1 on stack means return from plot, 0 return from line
    pha

dopixel:
    txa                ; Test color parameter
    beq stbitmask      ; Bits for parameter 0
    lda #$FF           ; Bits for parameter 1
    dex                ; dex should give us 0 if color=1
    beq stbitmask      ;
    lda #$55           ; Bits for parameter 2
    dex
    beq stbitmask    
    lda #$AA           ; Bits for parameter 3 or higher
stbitmask:
    sta $FD            ; Store it to unused zero page location ($FD) initially keep BITMASK
    lda #00            ; 
    sta $F7            ; Put 0 to lower byte of product pixel character row by 64
    lda $FC            ; Get Y coordinate
    tax
    and #7             ; Lower 3 bits in Y coordinate determine scan line in 8x8 block
    tay                ; Keep in Y
    txa
    lsr                ; Y/2
    lsr                ; Y/4
    lsr                ; Y/8. $F9 will contain number of pixel character row (8 scan lines)
    sta $F9            ; We have already 256xpixel character row     
    lsr                ; Now divide higher byte by 2
    ror $F7            ; shift lower
    lsr                ; And in A we have higher byte of 64 x pixel character row
    ror $F7            ; In $F7 we have higher byte of 64 x pixel character row
    adc $F9            ; Add higher byte with higher byte of pixel character Row * 256
    adc #$E0           ; Add high byte of bitmap base, we do not need CLC here, max value is $1F
    sta $F8            ; $F7 and F8 now contain address of pixel character row
    lda $FA            ; X coordinate, lower byte
    tax                ; keep it in index register X as well
    and #$F8           ; mask lower 3 bits, 8 pixels difference is 8 bytes
                       ;
    adc $F7            ; Add X AND #$F8 to pixel row addres lower byte, carry seems 0 here
    sta $F7            ; store to pixel row address
    lda $FB            ; Get X higher byte
    adc $F8            ; Add with pixel character row address higher byte
    sta $F8            ; Now we have pixel character address in ($F7,$F8)

    lda $D016          ; Load Screen control register #2
    and #$10           ; Check bit 4 for multicolor
    beq hires          ; If zero, go to hires handling
    txa                ; Otherwise, handle multicolor
    and #$06           ; Masking for multicolor (X coordinate)
    tax                ; Store pixel number (0-7) to X
    lda #$80           ; Preset A for multicolor
    bne setsec         ; A is not zero, always branches
hires:
    txa                ; Handle hires
    and #$07           ; Masking for hires (X coordinate)
    tax                ; Store pixel number (0-7) to X
    lda #$00           ; Preset A for hires
setsec:
    inx                ; Loop executes pixel number+1 times
    sec                ; Set carry
shpix:
    ror a              ; Float single bit
    dex                ; Decrement loop counter 
    bne shpix          ; And repeat
    tax                ; Bit is ready form masking and stuffing
    sei                ; Disable interrupt. We need to switch off KERNAL ROM
    lda $01            ; 6510 register 1 defines selected memory.
    pha                ; Remember previous memory configuration
    lda #$34           ; Switch off ROM, turn on RAM, because we need to read from RAM
    sta $01            ; Write to 6510 register 1 to redefine memory

    txa                ; Has zeros except at bits that need to be changed
    and $FD            ; Setup earlier HLHLHLHL or CCCCCCCC
    sta $03
    txa                ; Has zeros except at bits that need to be changed
    eor #$FF           ; Flip
    and ($F7),Y        ; Read from pixel character address plus relative row 
    ora $03
    sta ($F7),Y        ; set the byte

    pla                ; restore memory configuration
    sta $01            ; Write to 6510 register 1 to redefine memory
    cli                ; Enable interrupt
    pla
    beq retline1       ; Entry point for LINE               

    jmp $A7AE          ; jump to main interpreter loop to check new command      
endplot:
.endif

.ifdef CMDGETPRINT

text:                  ; !GETPRINT mode 0 copy caps, 1 over caps, 2 copy lowercase, 3 over lowercase
    cmp #$DC           ; Hash of GETPRINT
    bne endtext        ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    txa
    ldx #$0
    lsr
    bcc setover
    dex
setover:
    stx $FE

    ldx #$D0
    lsr
    bcc selfont
    ldx #$D8
selfont:
    stx $02

    lda #0
    sta $F8
    sta $FC
    lda $288           ; (F8,F9) Address of text screen
    sta $F9
    lda #$E0           ; (FC,FD) Address of bitmap
    sta $FD
    sei                ; Disable interrupt. We need to switch off KERNAL ROM and I/O devices
    lda $01            ; 6510 register 1 defines selected memory.
    pha                ; Remember previous memory configuration
    lda #$31           ; Put character ROM in address space, switch off KERNAL ROM
    sta $01            ; Write to 6510 register 1 to redefine memory
copychar:              ; Start from beginning of textual video memory
    lda #0
    sta $FB            ; High byte of char definition is zero
    ldy #0             ; Get text character
    lda ($F8),y        ; Get screen code of PET character
    asl                ; Multiply it by 8, high byte will be in $FB
    rol $FB            ; x2
    asl
    rol $FB            ; x4
    asl
    rol $FB            ; x8
    sta $FA
    lda $FB            ; Store high byte of 8x screen code
    clc                ; Add with start of switched in font
    adc $02

    sta $FB            ; (FA,FB) contains address of character 8x8 definition
    ldx #$08
    ldy #0
charloop:              ; It is good luck that C64 organizes video memory per character scanline
    lda ($FC),y        ; Take from video RAM
    and $FE            ; AND with keeper
    ora ($FA),y        ; Merge font definition
    sta ($FC),y        ; Put to memory
    inc $FA            ; Increment pointer to font low byte
    bne incbitmap      ; If not flipped skip high
    inc $FB            ; Increment pointer to font high byte
incbitmap:
    inc $FC            ; Increment pointer to bitmap low byte
    bne nextscanline   ; If not flipped skip high
    inc $FD            ; Increment pointer to bitmap high byte
nextscanline:
    dex                ; decrement counter
    bne charloop       ; If not repeat plotting                       
    inc $F8            ; Increment pointer to text video low byte
    bne chklast        ; If not flipped skip high
    inc $F9            ; Increment pointer to text video high byte
chklast:               ; Have we finished video memory?
    lda $FD
    cmp #$FF           ; FF40 is a last character in bitmap
    bne copychar       ; repeat loop if they are not both
    lda $FC
    cmp #$40
    bne copychar       ; repeat loop if they are not both
    pla                ; restore memory configuration
    sta $01            ; Write to 6510 register 1 to redefine memory
    cli                ; Enable interrupt
    jmp $A7AE          ; jump to main interpreter loop to check new command      
endtext:
.endif

.ifdef CMDSQRCLR

attrib:                ; !SQRCLR x,y,color1,color2,color3 graph0: graph1: u: o:
    cmp #$11           ; Hash of SQRCLR
    bne endattrib      ; if not try other letter
    jsr $B79B          ; get byte parameter, with advancing scan, result in X
    cpx #40            ; 0-39
    bcs attriberr     ; if greater or equal carry=1
    stx $FB            ; x coordinate
    jsr $B7F1          ; Check for comma then get byte parameter into X
    lda #0
    sta $FD
    stx $FC            ; y coordinate
    txa
    cmp #25            ; 0-24
    bcs attriberr      ; if greater or equal carry=1

    asl                ; y*2
    asl                ; y*4
    adc $FC            ; y*5
    asl
    rol $FD            ; y*10
    asl
    rol $FD            ; y*20
    asl
    rol $FD            ; y*40
    adc $FB            ; y*40+X  (input carry is zero at this point)
    sta $FC
    bcc getattrcol1
    inc $FD
getattrcol1:
    jsr $B7F1          ; Check for comma then get byte parameter into X
    stx $FB            ; We do not need X any longer, so put lower attribute nibble
    jsr $B7F1          ; Check for comma then get byte parameter into X
    txa                ; Put second color into upper nibble
    asl
    asl
    asl
    asl
    ora $FB            ; Combine nibbles
    sta $FB   
    jsr $B7F1          ; Check for comma then get byte parameter into X
    ldy #0
    clc
    lda $FD
    adc #$CC           ; CC00+addr
    sta $FD
    lda $FB
    sta ($FC),y        ; Update values in screen memory
    lda $FD
    adc #$0C           ; D800+addr
    sta $FD   
    txa
    sta ($FC),y        ; Update value in color memory
    
    jmp $A7AE          ; jump to main interpreter loop to check new command  
attriberr:
    jmp $B248          ; Illegal quantity error
    
endattrib:

.endif

.ifdef CMDSPOS

jump:                  ; !SPOS num,x,y,visible
    cmp #$5F           ; Hash of SPOS
    bne endjump        ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    txa                ; Take sprite number, but just 3 lowest bits
    and #7             ; masking
    tax                ; store sprite number
    asl
    sta $F9            ; F9 contains 2xsprite num

    inx                ; Loop executes pixel number+1 times
    lda #$00           ; carry will float around acumulator
    sec                ; Set carry
sprx:
    rol a              ; Float single bit
    dex                ; Decrement loop counter 
    bne sprx           ; And repeat
    sta $FA            ; Bit is ready for register $D010 and $D015

    jsr $AEFD          ; check for comma
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI 
    ldx $F9            ; 2x sprite num
    cmp #0             ; If hi bit=0 of x
    beq sm             ; If it is 0, we are reseting and mask is 00
    lda #$FF           ; We will not check X , simply out of screen
sm:
    sta $FD            ; Keep mask 00 or FF
    tya                ; Low byte is in y
    sta $D000,x        ; Sprite x=$D000+2*x
                       
    lda $D010          ; Read from high bit register
    eor $FD            ; BITMASK=FF for seting, 0 for clearing bit
    and $FA            ; AND with sprite bit
    eor $D010          ; XOR with current byte
    sta $D010          ; Set highest x bit $D010

    jsr $B7F1          ; Check for comma then get byte parameter into X
    txa
    ldx $F9
    sta $D001,x        ; Sprite y=$D001+2*x

    jsr $B7F1          ; Check for comma then get byte parameter into X
                       ; Sprite visible bit $D015
    cpx #0             ; If not visible
    beq sv             ; skip mask define
    lda #$FF           ; Mask for setting
sv:
    sta $FD
    lda $D015          ; Read sprite visible register
    eor $FD            ; BITMASK=FF for seting, 0 for clearing bit
    and $FA            ; AND with sprite bit
    eor $D015          ; XOR with current byte
    sta $D015          ; set the byte

    jmp $A7AE          ; jump to main interpreter loop to check new command      

endjump:
.endif

.ifdef CMDSDEF

spritedef:             ; !SDEF num,addr,color
    cmp #$3C           ; hash of SDEF
    bne endspritedef   ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    txa                ; Take sprite number, but just 3 lowest bits
    and #7             ; masking
    tax                ; store sprite number
    sta $F9            ; F9 contains sprite num

    lda #$F8           ; Last 8 bytes of screen memory are sprite pointer
    sta $FB            ; So lower byte of screen memory address is FB
    lda $DD00          ; Bits 15-14 of screen memory are inverted two lowest bits of VIC base
    and #$03           ; Ignore other bits of DD00
    eor #$03           ; Invert them
    sta $FC            ; Temporary store
    lda $D018          ; Read VIC Memory Control Register 
    and #$F0           ; In the higher nibble we have bits 13-10 of the Screen address
    lsr $FC            ; Push topmost bits into accumulator
    ror
    lsr $FC
    ror
    clc                ; As sprite pointers are at end of screen RAM
    adc #3             ; Increment it by 3 (lower byte is $FB , so almost a kilobyte)
    sta $FC            ; FC-FD now keeps address of sprite pointer
    jsr $AEFD          ; check for comma
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI 
        
    sty $FA            ; Addres of sprite relative from VIC as (ADDR MOD 16384)/64
    asl $fa            ; The same effect is shifting address to left twice
    rol
    asl $fa
    rol                ; A contains sprite pointer
    ldy $F9            ; Get sprite number
    sta ($FB),y        ; And do it

    jsr $B7F1          ; Check for comma then get byte parameter into X
    txa
    ldx $F9            ; Get sprite number
    sta $D027,x        ; color = $D027+spriteno
    jmp $A7AE          ; jump to main interpreter loop to check new command      
endspritedef:
.endif

.ifdef CMDIFCLOSE

kicksprite:            ; !IFCLOSE num
    cmp #$B7           ; Hash of IFCLOSE
    bne endkicksprite  ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    txa                ; Take sprite number, but just 3 lowest bits
    and #7             ; masking
    tax                ; store sprite number
    inx                ; Loop executes pixel number+1 times
    lda #$FF           ; carry will float around acumulator
    clc                ; Set carry
kicksprx:
    rol a              ; Rotate single bit
    dex                ; Decrement loop counter 
    bne kicksprx       ; And repeat
    ora $d01e
    ora $d01f
    cmp #$FF
    bne nokick
kick:
    lda #$01
    ora $90
    sta $90
    jmp $A7AE          ; jump to main interpreter loop to check new command          
nokick:
    lda #$FE
    and $90
    sta $90
    jmp $A7AE          ; jump to main interpreter loop to check new command      
endkicksprite:
.endif

.ifdef CMDEXPAND

xpandsprite:           ; !EXPAND num,DIR,val  
                       ; DIR= 0: enable, 1: X, 2: Y, 3: priority , 4:multicolor
    cmp #$2A           ; Hash of EXPAND
    bne endxpandsprite ; if not try other letter
    jsr $B79B          ; scan and get byte parameter, result in X
    txa                ; Take sprite number, but just 3 lowest bits
    and #7             ; masking
    tax                ; store sprite number
    inx                ; Loop executes pixel number+1 times
    lda #$00           ; carry will float around acumulator
    sec                ; Set carry
xpdsprx:
    rol a              ; Float single bit
    dex                ; Decrement loop counter 
    bne xpdsprx        ; And repeat
    sta $FA            ; Bit is ready for registers $D01E and $D01F
    jsr $B7F1          ; Check for comma then get byte parameter into X
    cpx #5             ; 0-4
    bcs xpnderr        ; if greater or equal carry=1, parameter overflow

    cpx #1             ; If DIR=1, register is at offset 8
    bne try3
    ldx #8
try3:
    cpx #3             ; If DIR=3, register is at offset 6
    bne try4
    ldx #6
try4:
    cpx #4             ; If DIR=4, register is at offset 7
    bne gotreg
    ldx #7
gotreg:                ; If DIR=2 or DIR=0, register is at offset DIR                       
    stx $FE            ; 0=ENABLE, 2=Y, 6=priority , 7=multicolor , 8=X
    jsr $B7F1          ; Check for comma then get byte parameter into X

    txa                ; To A register
    beq xpd1           ; if zero skip mask define
    lda #$FF           ; Mask for setting
xpd1:
    sta $FD            ; BITMASK
    ldx $FE            ; Set offset to X

    lda $D015,X        ; Read sprite X expand register
    eor $FD            ; BITMASK=FF for seting, 0 for clearing bit
    and $FA            ; AND with sprite bit
    eor $D015,X        ; XOR with current byte
    sta $D015,X        ; set the byte
    jmp $A7AE          ; jump to main interpreter loop to check new command      
xpnderr:
    jmp $B248          ; Illegal quantity error
endxpandsprite:
.endif

.ifdef CMDWAITFOR

interval:              ; !WAITFOR time in ms
    cmp #$A6           ; Hash of WAITFOR
    bne endinterval    ; if not try other letter
    jsr $0073          ; Scan for next symbol
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI
    sty $FC            ; Zero page location ($FC-$FD) contains counter
    sta $FD
loop:
    jsr $EEB3          ; 1ms delay, KERNAL call
    lda $FC            ; Check high byte of the counter               
    bne declow         ; If it is not zero, it is enough to change just lower byte of counter
    lda $FD            ; High byte is zero at this point, so if the lower is zero as well
    beq finishloop     ; then finish the loop
    dec $FD            ; lower byte is zero, not yet finished, decrement higher byte
declow:
    dec $FC            ; Decrement the lower byte
    clc
    bcc loop           ; Jump back to loop
finishloop:
    jmp $A7AE          ; jump to main interpreter loop to check new command      
endinterval:
.endif

.ifdef CMDDPOKE

dpoke:                 ; !DPOKE addr,val
    cmp #$1F           ; Hash of DPOKE
    bne enddpoke       ; if not try other letter
    jsr $0073          ; Scan for next symbol
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI
    sty $FB            ; Store it to unused zero page location ($FB-$FC) 
    sta $FC
    jsr $AEFD          ; check for comma
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI 
    pha
    tya
    ldy #0
    sta ($FB),y
    pla
    iny
    sta ($FB),y
    jmp $A7AE          ; jump to main interpreter loop to check new command
enddpoke:
.endif

.ifdef CMDMPOKE

memory:                ; !MPOKE from,to,len
    cmp #$31           ; Hash of MPOKE
    bne endmemory    ; if not try other letter
    jsr $0073          ; Scan for next symbol
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI
    sty $FB            ; Zero page location ($FB-$FC) keeps from address
    sta $FC
    jsr $AEFD          ; check for comma
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI
    sty $FD            ; Zero page location ($FD-$FE) keeps to address
    sta $FE
    jsr $AEFD          ; check for comma
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI
    sty $02            ; Keep last page size in memory
    ldy #0             ; Set Y counter to zero, copy in 256 bytes blocks
    tax                ; Use X as counter of pages
    beq mcopylastpg    ; But if it is already zero, skip first part
mainmlp:               ; Loop that copyes 256 bytes blocks
    lda ($FB),y        ; Take source byte
    sta ($FD),y        ; Store to destination          
    iny                ; Increment small pointer
    bne mainmlp        ; Wrapped to zero?
    inc $FC            ; Increment higher bytes of source pointer
    inc $FE            ; And destination pointer
    dex                ; But decrement block counter
    bne mainmlp        ; And repeat until zero
mcopylastpg:           ; Remaining bytes
    ldx $02            ; X is now byte counter
    beq mcopyend       ; If it is yero we have finished
lastpglp:              ; Last block loop
    lda ($FB),y        ; Take source byte
    sta ($FD),y        ; Store to destination byte
    iny                ; increment small pointer
    dex                ; Decrement byte counter
    bne lastpglp       ; Until zero
mcopyend:
    jmp $A7AE          ; jump to main interpreter loop to check new command      
endmemory:
.endif

.ifdef CMDSTOPON

yell:                  ; !STOPON x   Yell program error at specified line
    cmp #$B2           ; Hash of STOPON
    bne endyell        ; if not try other letter
    jsr $0073          ; Scan for next symbol
    jsr $AD8A  	       ; Get Float parameter
    jsr $B7F7          ; Convert to 2 byte integer, Y=lo, A=HI
    sty $02A7          ; Store it to unused location $02A7
    sta $02A8

wheraiam:              ; Finding program counter
    sei                ; Disable interrupt
    jsr $FD0F          ; Call RTS
here:                  ; Interrupt is disabled, so stack still keeps this address -1
    tsx                ; Using X register
    ldy $100,x         ; Get higher byte
    dex                ; Decrement
    lda $100,x         ; Get lower byre
    cli                ; A low,Y hi pointing to hereis-1
    sec                ; So add 1
    adc #(handler-here); plus difference and point to relocatible error handler
    sta $0300          ; Store lower byte to ERR vec
    bcc hivec          ; If carry
    iny                ; then increase higher byte
hivec:
    sty $0301          ; Store higher byte to ERR vec 
    jmp $A7AE          ; jump to main interpreter loop to check new command      

handler:
    php                ; Save status word, because ldx was previous command
    lda #$8B           ; Normal Error vector will be restored
    sta $300           ; Low address of $E38B
    lda #$E3
    sta $301           ; High address byte
    plp                ; Now restore status word
    bpl doonerror      ; Highest bit indicates, no error
    jmp $E38B          ; Just print ready READY
doonerror:
    stx $030D          ; Error code
    lda $02a7          ; line lo -poke new value, maybe replace with private location?
    sta $14            ; Area for GOTO command
    lda $02a8          ; line HI -poke new value
    sta $15            ; Area for GOTO command
    ldx #$FA           ; Clear stack
    txs                ; To stack
    lda #$A7           ; Just after JSR $A7ED  go interpret BASIC code from BASIC execute pointer
    pha
    lda #$E9
    pha
    jmp $A8A3          ; Execute GOTO 
endyell:
.endif

.ifdef CMDRESTORETO

restore:               ; RESTORETO line restore to specified DATA line 
    cmp #$BD           ; Hash of RESTORETO
    bne endrestore     ; if not try other letter
    jsr $0073          ; Scan for next symbol
    JSR $AD9E          ; Evaluate expression
    JSR $B7F7          ; Convert to temporary integer
    JSR $A613          ;   search BASIC for temporary integer line number
    BCS dorestore      ; If Found dskip to next
    LDX #$11           ; Undefined statement
    JMP $A437          ; ERROR
dorestore:
    LDA $5F            ; Read number from temporary register
    SBC #$01           ; Decrement it
    STA $41            ; DATA pointer low
    LDA $60            ; Read number from temporary register
    SBC #$00           ; Decrement higher byte
    STA $42            ; DATA pointer hi
    jmp $A7AE          ; jump to main interpreter loop to check new command      
endrestore:

.endif

;****** ADD NEW COMMANDS HERE



endcomm:
    jmp $AF08  ; Syntax error

.ifdef LOWVER
filler:
dist=((filler-basic)/$100+1)*$100-(filler-basic)
    .repeat dist  
    .byte 0
    .endrep
    .word 0          ;basic line end
    .byte 0
.endif

.ifdef HIGHVER
endcode:
    .word 0          ;basic line end
    .byte 0
.endif

